/* oozext.c -- extracts files from archive */

/* Main extraction module.  This file is public domain.

                                   -- Rahul Dhesi 1991/07/07

The function fixfname() is currently defined to be empty but may be
activated if needed to fix filename syntax to be legal for your
computer system.  Look near the end of this file for instructions and
a sample implementation of fixfname().  Currently, fixfname() is used
only if the symbol FIXFNAME is defined.
*/

#include "booz.h"
#include "zoo.h"
#include <stdio.h>

extern unsigned int crccode;

int needed ();
extern char long_name[];

int oozext (char *zoo_path, char *option, int argc, char *argv[])

{
FILE *zoofile;                            /* open archive */
FILE *this_file;                          /* file to extract */
long next_ptr;                            /* pointer to within archive */
struct zoo_header zoo_header;             /* header for archive */
int status;                               /* error status */
struct direntry direntry;                 /* directory entry */
int all = 0;                              /* overwrite all? */
static char vermsg[] = "A higher version of Ooz is needed to extract ";
int exitstat = 0;                         /* return code */
int first_time = 1;
static char *month_list="000JanFebMarAprMayJunJulAugSepOctNovDec";

{

#ifndef ARM
   /* If no dot in name, add ".zoo" extension */
   char *p;
   p = zoo_path;
   while (*p != '\0' && *p != '.')
      p++;
   if (*p != '.') {  /* no dot found */
      p = malloc (strlen (zoo_path) + 5);
      if (p == (char *) 0)
         memerr();
      strcpy (p, zoo_path);
      strcat (p, ".zoo");
      zoo_path = p;
   }
#endif
}

zoofile = OPEN(zoo_path);

if (zoofile == NULL)
   prterror ('f', "Could not open ", zoo_path, "\n");

/* read header */
rd_zooh (&zoo_header, zoofile);
fseek(zoofile, zoo_header.zoo_start, 0); /* seek to where data begins */

while (1) {
   rd_dir (&direntry, zoofile);

         if (direntry.zoo_tag != ZOO_TAG)
         prterror ('f', "Bad entry in archive\n", 
            ((char *) 0), ((char *) 0));
   if (direntry.next == 0L) {                /* END OF CHAIN */
      break;                                 /* EXIT on end of chain */
   }
   next_ptr = direntry.next;                 /* ptr to next dir entry */

      /* See if this file is wanted */
      if (!needed (long_name, argc, argv)) {
         goto loop_again;
      }

  /* If list needed, give information and loop again */
   if (*option == 'l') {
      char outstr[80];
      char buf[20];
      int year, month, day, hours, min, sec;
      int size_factor;
      size_factor = cfactor (direntry.org_size, direntry.size_now);
   
      year  =  ((unsigned int) direntry.date >> 9) & 0x7f;
      month =  ((unsigned int) direntry.date >> 5) & 0x0f;
      day   =  direntry.date        & 0x1f;
      
      hours =  ((unsigned int) direntry.time >> 11)& 0x1f;
      min   =  ((unsigned int) direntry.time >> 5) & 0x3f;
      sec   =  ((unsigned int) direntry.time & 0x1f) * 2;

      if (first_time) {
         putstr ("Length    CF  Size Now  Date      Time\n");
         putstr ("--------  --- --------  --------- --------\n");
         first_time = 0;
      }
      strcpy (outstr, itoa(' ', direntry.org_size, buf, 9));
      strcat (outstr, itoa(' ',(long) size_factor, buf, 5));
      strcat (outstr, "% ");
      strcat (outstr, itoa(' ',direntry.size_now, buf, 9));
      strcat (outstr, "  ");
      strcat (outstr, itoa(' ',(long) day, buf, 3));
      strcat (outstr, " ");
      strncat (outstr, &month_list[month*3], 3);
      strcat (outstr, " ");
      if (day && month)
         strcat (outstr, itoa(' ',(long) (year+80) % 100, buf, 3));
      else
         strcat (outstr, itoa(' ',0L, buf, 3));
      strcat (outstr, " ");
      strcat (outstr, itoa('0',(long) hours, buf, 3));
      strcat (outstr, ":");
      strcat (outstr, itoa('0',(long) min, buf, 3));
      strcat (outstr, ":");
      strcat (outstr, itoa('0',(long) sec, buf, 3));
      strcat (outstr, "  ");
      strcat (outstr, long_name);
      strcat (outstr, "\n");
      putstr (outstr);

#ifdef COMMENT
      printf ("%8lu %3u%% %8lu  %2d %-.3s %02d %02d:%02d:%02d  ",  
               direntry.org_size, 
               size_factor, direntry.size_now, 
               day, &month_list[month*3], 
               (day && month) ?  (year+80) % 100 : 0,
               hours, min, sec);
      printf ("%s\n", long_name);
#endif

     goto loop_again;
   }


   if (direntry.major_ver > 2 ||
         (direntry.major_ver == 2 && direntry.minor_ver > 1)) {
      prterror ('e', vermsg, long_name, "\n");
      goto loop_again;
   }

#ifdef FIXFNAME
   /* Make the filename syntax acceptable to the host system */
         fixfname (long_name);
#endif


   /* See if this file already exists */

   if (*option != 't' && !all) {
      this_file = OPEN(long_name);
      if (this_file != NULL) {
         char ans[2];
         char ans2[2];
         fclose(this_file);
   
         do {
            prterror ('m', "Overwrite ", long_name, " (Yes/No/All)? ");
            fread(ans, 1, 1, stdin);
            do {
               fread(ans2, 1, 1, stdin);
            } while (*ans2 != '\n');
         }  while (*ans != 'y' && *ans != 'Y' && 
                   *ans != 'n' && *ans != 'N' &&
                   *ans != 'a' && *ans != 'A');    
   
         if (*ans == 'a' || *ans == 'A')
            all++;
         if (*ans == 'n' || *ans == 'N') {
            prterror ('m', "Skipping ", long_name, "\n");
            goto loop_again;
         }
      }
   }

   if (*option == 't')
      this_file = NULL;
         else {
                        int retry=2;
                        do {
                                retry--;
                                this_file = CREATE(long_name);
                                if( !this_file ) build_dirs( long_name );
                        } while( !this_file && retry );
         }

   if (*option != 't' && this_file == NULL) {
      prterror ('e', "Could not open ", long_name, " for output.\n");
   } else {
      fseek(zoofile, direntry.offset, 0);
      crccode = 0;      /* Initialize CRC before extraction */
      putstr(long_name);
      putstr(" ");

      if (direntry.packing_method == 0)
                                 status = getfile(zoofile, this_file, direntry.size_now, long_name);
                        else if (direntry.packing_method == 1)
                                 status = lzd(zoofile, this_file, long_name); /* uncompress */
                        else if (direntry.packing_method == 2)
                                 status = lzh_decode(zoofile, this_file, long_name); /* uncompress */
                        else
                                 prterror ('e', vermsg, long_name, "\n");
                        if (status != 0) {
#ifdef ARM
                                           remove(long_name);
#else
                                           unlink(long_name);
#endif
                                 if (status == 1) {
                                                memerr();
                                 } else
                                                prterror ('e', "I/O error writing ", long_name, "\n");
                        } else {
         if (direntry.file_crc != crccode) {
            putstr ("<--\007WARNING:  Bad CRC.\n");
            exitstat = 1;
         } else {
            putstr ("\n");
         }
      } /* end if */
   } /* end if */

   if (*option != 't' && this_file)
      fclose(this_file);

loop_again:
   fseek(zoofile, next_ptr, 0); /* ..seek to next dir entry */
} /* end while */
fclose(zoofile);
exit (exitstat);
} /* end oozext */

/*
Function fixfname() fixes the syntax of the supplied filename so it is
acceptable to the host system.  We call the standard string function 
strchr(s, c) which searches a string s for a character c and returns
a pointer to it or returns NULL if not found.  The scheme below
strips the 8th bit and changes control characters to corresponding 
printable characters (e.g.  ^A becomes A).

As supplied, fixfname() is activated only if the symbol FIXFNAME
is defined.
*/

#ifdef ARM
int fixfname( char *fname )
{
        char *p;
        char *last_dot;
        char tmp[100];

        while(p=strchr(fname,'$'))    *p='_';

        last_dot=strrchr(fname,'.');
        if( last_dot ) {
                        while( (p=strchr(fname,'.'))!=last_dot )  *p='_';
                        if  ((p=strrchr (fname,'/')) && last_dot < p)
                            *last_dot = '_';   /* for version number 3.1 */
        }


        if( strcmp( last_dot, ".c" )==0 || strcmp( last_dot, ".h" )==0 ) {
                p=strrchr( fname, '/' );
                if( !p ) {
                        p=fname;
                } else {
                        p++;
                }
                strcpy( tmp, p );
                *strchr( tmp, '.' )='\0';
                strcpy( p, last_dot+1 );
                strcat( p, "." );
                strcat( p, tmp );
        } else {
                *last_dot='_';
        }
        while( p=strchr( fname, '/' ) ) *p='.';
        strrchr(fname,'.')[10]='\0';

}

int build_dirs( char *name )
{
        char *p,*pe;
        char cmd [256];

        for( p=name, pe=strchr(p,'.') ; pe ; p=pe+1, pe=strchr(p,'.') ) {
                *pe='\0';
                sprintf (cmd,"*cdir %s\n",name);
                system  (cmd);
                *pe='.';
        }
}
#endif

#ifdef ATARI
int fixfname( char *fname )
{
        char *p;
        char *last_dot=NULL;

        for( p=fname ; *p ; p++ ) {
                if( *p=='/' ) {
                        last_dot=NULL;
                        *p='\\';
                }
                if( *p=='.' ) {
                        if( last_dot ) *last_dot='_';
                        last_dot=p;
                }
        }
        if( strcmp( fname+strlen(fname)-11, "$compile1.P" )==0 ) {
                strcpy( fname+strlen(fname)-11, "$compil1.P" );
        }

        if( strcmp( fname+strlen(fname)-12, "$asmpass11.P")==0 ) {
                strcpy( fname+strlen(fname)-12, "$asmpas11.P" );
  }

        if( strcmp( fname+strlen(fname)-12, "$asmpass21.P")==0 ) {
                strcpy( fname+strlen(fname)-12, "$asmpas21.P" );
  }
}

#ifdef __TURBOC__
#include <tos.h>
#else
#include <osbind.h>
#endif

int build_dirs( char *name )
{
        char *p,*pe;

        for( p=name, pe=strchr(p,'\\') ; pe ; p=pe+1, pe=strchr(p,'\\') ) {
                *pe='\0';
                Dcreate( name );
                *pe='\\';
        }
}
#endif

#ifdef AMIGA
int build_dirs( char *name )
{
        char *p,*pe;

        for( p=name, pe=strchr(p,'/') ; pe ; p=pe+1, pe=strchr(p,'/') ) {
                *pe='\0';
                mkdir( name );
                *pe='/';
        }
}
#endif
